-------------------------------------------------------------------------------
-- Lorewalkers by Lockslap, US-Bleeding Hollow
-- <Borderline Amazing>, http://ba-guild.com
-------------------------------------------------------------------------------
Lorewalkers	= LibStub("AceAddon-3.0"):NewAddon("Lorewalkers", "AceEvent-3.0", "AceConsole-3.0")
if not TomTom then return end

-- libaries
local L					= LibStub("AceLocale-3.0"):GetLocale("Lorewalkers", false)
local Astrolabe			= DongleStub("Astrolabe-1.0")
local AceConfig			= LibStub("AceConfig-3.0")
local AceConfigDialog	= LibStub("AceConfigDialog-3.0")
local AceConfigRegistry	= LibStub("AceConfigRegistry-3.0")

-- setup some globals
Lorewalkers.name 	= GetAddOnMetadata("Lorewalkers", "Title")
Lorewalkers.version	= GetAddOnMetadata("Lorewalkers", "Version")

local waypoints, zones, db	= {}, {}, {}

local enabledString		= ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString	= ("|cffff0000%s|r"):format(L["Disabled"])
local repStandings 		= {
	[1]	= L["Hated"],
	[2]	= L["Hostile"],
	[3]	= L["Unfriendly"],
	[4]	= L["Neutral"],
	[5]	= L["Friendly"],
	[6]	= L["Honored"],
	[7]	= L["Revered"],
	[8]	= L["Exalted"],
}

-- locale information
local locales = {
	["deDE"]	= L["German"],
	["enGB"]	= L["British English"],
	["enUS"]	= L["American English"],
	["esES"]	= L["Spanish (European)"],
	["esMX"]	= L["Spanish (Latin American)"],
	["frFR"]	= L["French"],
	["koKR"]	= L["Korean"],
	["ruRU"]	= L["Russian"],
	["zhCN"]	= L["Chinese (Simplified)"],
	["zhTW"]	= L["Chinese (Traditional)"],
}
local supportedLocales = {
	"enUS",	
	"enGB",
	"frFR",	-- translations by zlika
	"zhTW",	-- translations by machihchung
}

-- options page
local fields = {"Title", "Version", "Author", "X-Category", "X-License", "X-Email", "Email", "eMail", "X-Website", "X-Credits", "X-Localizations", "X-Donate", "X-Paypal"}
local fieldName = "|cff1784d1%s:|r"
local defaults 		= {
	char = {
		debug		= UnitName("player") == "Lockslap" and true or false,	-- i need debugging by default
		enabled 	= true,
		arrow		= true,
		minimap		= true,
		worldmap	= true,
		criterias	= {}
	}
}
local options = {
	type	= "group",
	args	= {
		general	= {
			type	= "group",
			name	= Lorewalkers.name,
			get		= function(info) return db[info[#info]] end,
			set		= function(info, value) 
				db[info[#info]] = value
				Lorewalkers:UpdateWaypoints()
			end,
			args	= {
				header = {
					type		= "header",
					order		= 1,
					name		= L["Version"] .. (": |cff99ff33%s|r"):format(Lorewalkers.version),
					width		= "full",
				},
				description = {
					type		= "description",
					order		= 2,
					name		= L["A simple addon to aid you with obtaining exalted with The Lorewalkers."],
					width		= "full",
				},
				enabled = {
					type		= "toggle",
					order		= 3,
					name		= L["Enable Lorewalkers"],
					desc		= L["Toggle the waypoints."],
				},
				debug = {
					type		= "toggle",
					order		= 4,
					name		= L["Debugging"],
					desc		= L["This will ignore if you have completed the achievements/criteria and load the waypoints.  I use this for fixing issues."],
				},
				dummy = {
					type		= "description",
					order		= 99,
					name		= "",
					width		= "full",
				},
				addTo = {
					type		= "group",
					order		= 100,
					guiInline	= true,
					name		= L["Add To"],
					disabled	= function() return not db.enabled end,
					args		= {
						arrow = {
							type		= "toggle",
							order		= 1,
							name		= L["Crazy Arrow"],
							desc		= L["Add waypoints to TomTom's crazy arrow."],
							get			= function() return db.arrow end,
							set			= function(_, value)
								db.arrow = value
								Lorewalkers:UpdateWaypoints()
							end,
						},
						minimap = {
							type		= "toggle",
							order		= 2,
							name		= L["Minimap"],
							desc		= L["Add waypoints to the minimap."],
							get			= function() return db.minimap end,
							set			= function(_, value)
								db.minimap = value
								Lorewalkers:UpdateWaypoints()
							end,
						},
						worldmap = {
							type		= "toggle",
							order		= 3,
							name		= L["World Map"],
							desc		= L["Add waypoints to the world map."],
							get			= function() return db.worldmap end,
							set			= function(_, value)
								db.worldmap = value
								Lorewalkers:UpdateWaypoints()
							end,
						},
					},
				},
			},
		},
	}
}

local function InArray(needle, haystack)
	if #haystack == 0 then return false end
	for i = 1, #haystack do
		if haystack[i] == needle then
			return true
		end
	end
	return false
end

local function RepStatus()
	-- id 1345 are the lorewalkers
	local _, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(1345)
	barValue = barValue - barMin
	barMax = barMax - barMin
	return barValue, barMax, repStandings[standing], math.floor(barValue / barMax * 100)
end

local function UpdateCriteria()
	for title, id in pairs(Lorewalkers.achievements) do
		for i = 1, GetAchievementNumCriteria(id) do
			local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
			if not db.criterias[description] then
				db.criterias[description] = {
					id			= id,
					num			= i,
					completed	= completed,
				}
			else
				db.criterias[description].completed = completed
			end
		end
	end
end

local function GetCriteriaCoordinates(criteria)
	for zone, waypoints in pairs(Lorewalkers.waypoints) do
		for name, info in pairs(waypoints) do
			if criteria == name then
				return zone, info.x, info.y
			end
		end
	end
	return false
end

function Lorewalkers:HandleChatCommand(args)
	local command, value = self:GetArgs(args, 2)
	local iconString = "\124T%s:12\124t"
	local completedString = "%s%s %s |cff1784d1%d/%d/%d|r"
	local notCompletedString = "%s%s |cffff0000%s|r"
	local criteriaCompleted = "    %d. %s: |cffffff00%s|r%s |cff00ff00%s|r"
	local criteriaNotCompleted = "    %d. %s: |cffffff00%s|r%s |cffff0000%s|r"
	local achievementPattern = "\124c%x+\124Hachievement:(%d-):.-\124h%[.-%]\124h\124r"
	
	if command == "toggle" then
		db.enabled = not db.enabled
		Lorewalkers:Print((L["Waypoints have been %s"]):format(db.enabled == true and enabledString or disabledString))
		if not db.enabled then
			self:ClearWaypoints()
		else
			UpdateCriteria()
			Lorewalkers:UpdateWaypoints()
		end
		
	elseif command == "config" then
		InterfaceOptionsFrame_OpenToCategory(self.configFrame)
		
	elseif command == "rep" or command == "reputation" then
		local exaltedString	= "%s |cff1784d1%d/%d %s (%d%%)|r %s |cff1784d1%s|r."
		local value, max, standing, percent = RepStatus()
		self:Print(exaltedString:format(L["You are"], value, max, standing, percent, L["with"], L["The Lorewalkers"]))
		
	elseif command == "criterias" or command == "criteria" then
		local id = 0
		if value:match(achievementPattern) then
			id = value:match(achievementPattern)
		elseif value:find(" ") then
			-- they provided a string, it must be in quotes
			if not Lorewalkers.achievements[value] then
				self:Print((L["Achievement |cffff0000\"%s\"|r doesn't exist."]):format(value))
				return
			else
				id = Lorewalkers.achievements[value]
			end
		else
			id = tonumber(value)
		end
		local link = GetAchievementLink(id)
		local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
		self:Print(completed == true and completedString:format(iconString:format(icon), link, L["Completed on"], month, day, year) or notCompletedString:format(iconString:format(icon), link, L["Incomplete"]))
		for i = 1, GetAchievementNumCriteria(id) do
			local description, _, completed, _, _, _, _, _, _, _ = GetAchievementCriteriaInfo(id, i)
			local zone, x, y = GetCriteriaCoordinates(description)
			if zone then
				print(completed == true and criteriaCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y), L["Completed"]) or criteriaNotCompleted:format(i, description, zone, ("(%.2f, %.2f)"):format(x, y), L["Incomplete"]))
			else
				print(completed == true and criteriaCompleted:format(i, description, "", "", L["Completed"]) or criteriaNotCompleted:format(i, description, "", "", L["Incomplete"]))
			end
		end
		
	elseif command == "achieve" or command == "achievements" then
		for _, id in pairs(Lorewalkers.achievements) do
			local link = GetAchievementLink(id)
			local _, _, _, completed, month, day, year, _, _, icon, _, _ = GetAchievementInfo(id)
			self:Print(completed == true and completedString:format(iconString:format(icon), link, L["Completed on"], month, day, year) or notCompletedString:format(iconString:format(icon), link))
		end
	
	elseif command == "debug" then
		db.debug = not db.debug
		Lorewalkers:Print((L["Waypoints have been %s"]):format(db.debug == true and enabledString or disabledString))
		self:UpdateWaypoints()
		
	elseif not command or command == "" or command == "help" or command == "?" then
		self:Print((L["Lorewalkers version |cff1784d1%.2f|r by |cff00ff00Lockslap|r"]):format(self.version))
		self:Print(L["Chat Command:  |cff1784d1/lorewalkers|r or |cff1784d1/lw|r"])
		print(L["    |cff1784d1config|r - Open Configuration Window"])
		print(L["    |cff1784d1help|r or |cff1784d1?|r - Print This Help Again"])
		print(L["    |cff1784d1toggle|r - Toggle the Waypoints"])
		print(L["    |cff1784d1rep|r or |cff1784d1reputation|r - Your Reputation with the Lorewalkers"])
		print(L["    |cff1784d1criteria|r or |cff1784d1criterias|r |cff1784d1<id or name>|r - Display Achievement Criteria"])
		print(L["    |cff1784d1achievements|r or |cff1784d1achieve|r - Lorewalkers Achievement Progress"])
	end
end

function Lorewalkers:UpdateWaypoints()
	local loaded, completed = 0, 0
	local completedString = "|cff00ff00(%d %s)|r"
	local zone = GetZoneText()
	
	if db.debug and GetCurrentMapContinent() == 6 and not self.waypoints[zone] then
		self:Print((L["No waypoints found for |cff1784d1%s|r."]):format(zone))
		
	elseif (db.debug or db.enabled) and self.waypoints[zone] and zones[zone] then	
		-- first clear all of our waypoints
		self:ClearWaypoints()
		for desc, point in pairs(self.waypoints[zone]) do
			if not db.criterias[desc] then
				self:Print((L["Waypoint |cff1784d1%s|r not found."]):format(desc))
				
			elseif db.debug or not db.criterias[desc].completed then	-- the criteria wasnt found, or it doesnt exist
				loaded = loaded + 1
				waypoints[#waypoints + 1] = TomTom:AddZWaypoint(6, zones[zone], point.x, point.y, point.name .. "\n" .. desc, false, db.minimap, db.worldmap, false, false, db.arrow)
			
				if db.debug then
					self:Print((L["Added waypoint |cff1784d1%s|r in |cff1784d1%s|r."]):format(desc, zone))
				end
				
			else
				completed = completed + 1
			end
		end
		
		-- print completed/loaded information
		if loaded > 0 then
			self:Print((L["Loaded |cff1784d1%d|r %s waypoints for |cff1784d1%s|r"]):format(loaded, completedString:format(completed, L["completed"]), GetZoneText()))
		else
			self:Print((L["You have completed all waypoints for |cff1784d1%s|r"]):format(GetZoneText()))
		end
	end
end

function Lorewalkers:ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end	
	waypoints = {}
	TomTom:ReloadWaypoints()
end

function Lorewalkers:HandleEvents(event, ...)
	if db.debug or db.enabled then
		self:UpdateWaypoints()
	end
end
Lorewalkers:RegisterEvent("ZONE_CHANGED_NEW_AREA", "HandleEvents")

function Lorewalkers:OnInitialize()
	-- load the config
	self.db = LibStub("AceDB-3.0"):New("LorewalkersDB", defaults)
	db = self.db.char
	
	-- load the achievement criteria
	UpdateCriteria()
	for id, name in pairs{ GetMapZones(6) } do
		zones[name] = id
	end
	
	-- setup tomtom
	TomTom.profile.arrow.enable = true
	TomTom.profile.arrow.arrival = 5
	
	-- do a locale check to inform the user
	local locale = GetLocale()
	if not InArray(locale, locales) then
		-- unsupported locale, notify user
		self:Print((L["Your locale |cff1784d1%s|r, %s, is not supported by this addon."]):format(locale, locales[locale]))
		self:Print(L["The addon will not function as intended until translations are completed."])
	end
end

function Lorewalkers:OnEnable()
	-- add the about group
	options.args.general.args.about = {
		type		= "group",
		order		= 101,
		name		= L["About"],
		guiInline	= true,
		args		= {},
	}
	local i = 1
	for _, field in pairs(fields) do
		local val = GetAddOnMetadata(self.name, field)
		if val then
			options.args.general.args.about.args[field] = {
				type	= "description",
				order	= i,
				width	= "normal",
				name	= fieldName:format(field:gsub("X%-", "")),
			}
			i = i + 1
			options.args.general.args.about.args[("%s_value"):format(field)] = {
				type	= "description",
				order	= i,
				width	= "double",
				name	= val,
			}
			i = i + 1
		end
	end

	-- setup config window
	AceConfigRegistry:RegisterOptionsTable(self.name, options)
	self.configFrame = AceConfigDialog:AddToBlizOptions(self.name, nil, nil, "general")

	self:RegisterChatCommand("lorewalkers", "HandleChatCommand")
	self:RegisterChatCommand("lw", "HandleChatCommand")
	
	if db.debug or db.enabled then
		self:UpdateWaypoints()
	end
end

function Lorewalkers:OnDisable()
	self:UnregisterChatCommand("lorewalkers")
	self:UnregisterChatCommand("lw")
	
	if #waypoints > 0 then
		self:ClearWaypoints()
	end
end